@@ -32,10 +32,10 @@ class SparkCredentials(Credentials):
     host: str
     method: SparkConnectionMethod
     schema: str
-    cluster: Optional[str]
-    token: Optional[str]
-    user: Optional[str]
     database: Optional[str]
+    cluster: Optional[str] = None
+    token: Optional[str] = None
+    user: Optional[str] = None
     port: int = 443
     organization: str = '0'
     connect_retries: int = 0
@@ -43,7 +43,17 @@ class SparkCredentials(Credentials):
 
     def __post_init__(self):
         # spark classifies database and schema as the same thing
-        self.database = self.schema
+        if (
+            self.database is not None and
+            self.database != self.schema
+        ):
+            raise dbt.exceptions.RuntimeException(
+                f'    schema: {self.schema} \n'
+                f'    database: {self.database} \n'
+                f'On Spark, database must be omitted or have the same value as'
+                f' schema.'
+            )
+        self.database = None
 
     @property
     def type(self):
@@ -267,21 +277,26 @@ class SparkConnectionManager(SQLConnectionManager):
                 break
             except Exception as e:
                 exc = e
-                if getattr(e, 'message', None) is None:
-                    raise dbt.exceptions.FailedToConnectException(str(e))
-
-                message = e.message.lower()
-                is_pending = 'pending' in message
-                is_starting = 'temporarily_unavailable' in message
-
-                warning = "Warning: {}\n\tRetrying in {} seconds ({} of {})"
-                if is_pending or is_starting:
-                    msg = warning.format(e.message, creds.connect_timeout,
-                                         i, creds.connect_retries)
+                if isinstance(e, EOFError):
+                    # The user almost certainly has invalid credentials.
+                    # Perhaps a token expired, or something
+                    msg = 'Failed to connect'
+                    if creds.token is not None:
+                        msg += ', is your token valid?'
+                    raise dbt.exceptions.FailedToConnectException(msg) from e
+                retryable_message = _is_retryable_error(e)
+                if retryable_message:
+                    msg = (
+                        f"Warning: {retryable_message}\n\tRetrying in "
+                        f"{creds.connect_timeout} seconds "
+                        f"({i} of {creds.connect_retries})"
+                    )
                     logger.warning(msg)
                     time.sleep(creds.connect_timeout)
                 else:
-                    raise dbt.exceptions.FailedToConnectException(str(e))
+                    raise dbt.exceptions.FailedToConnectException(
+                        'failed to connect'
+                    ) from e
         else:
             raise exc
 
@@ -289,3 +304,15 @@ class SparkConnectionManager(SQLConnectionManager):
         connection.handle = handle
         connection.state = ConnectionState.OPEN
         return connection
+
+
+def _is_retryable_error(exc: Exception) -> Optional[str]:
+    message = getattr(exc, 'message', None)
+    if message is None:
+        return None
+    message = message.lower()
+    if 'pending' in message:
+        return exc.message
+    if 'temporarily_unavailable' in message:
+        return exc.message
+    return None
