@@ -2,6 +2,7 @@ import unittest
 from unittest import mock
 
 import dbt.flags as flags
+from dbt.exceptions import RuntimeException
 from agate import Row
 from pyhive import hive
 from dbt.adapters.spark import SparkAdapter, SparkRelation
@@ -72,11 +73,11 @@ class TestSparkAdapter(unittest.TestCase):
             connection.handle  # trigger lazy-load
 
             self.assertEqual(connection.state, 'open')
-            self.assertNotEqual(connection.handle, None)
+            self.assertIsNotNone(connection.handle)
             self.assertEqual(connection.credentials.cluster, '01234-23423-coffeetime')
             self.assertEqual(connection.credentials.token, 'abc123')
             self.assertEqual(connection.credentials.schema, 'analytics')
-            self.assertEqual(connection.credentials.database, 'analytics')
+            self.assertIsNone(connection.credentials.database)
 
     def test_thrift_connection(self):
         config = self._get_target_thrift(self.project_cfg)
@@ -92,20 +93,20 @@ class TestSparkAdapter(unittest.TestCase):
             connection.handle  # trigger lazy-load
 
             self.assertEqual(connection.state, 'open')
-            self.assertNotEqual(connection.handle, None)
+            self.assertIsNotNone(connection.handle)
             self.assertEqual(connection.credentials.schema, 'analytics')
-            self.assertEqual(connection.credentials.database, 'analytics')
+            self.assertIsNone(connection.credentials.database)
 
     def test_parse_relation(self):
         self.maxDiff = None
         rel_type = SparkRelation.get_relation_type.Table
 
         relation = SparkRelation.create(
-            database='default_database',
             schema='default_schema',
             identifier='mytable',
             type=rel_type
         )
+        assert relation.database is None
 
         # Mimics the output of Spark with a DESCRIBE TABLE EXTENDED
         plain_rows = [
@@ -117,7 +118,7 @@ class TestSparkAdapter(unittest.TestCase):
             ('dt', 'date'),
             (None, None),
             ('# Detailed Table Information', None),
-            ('Database', relation.database),
+            ('Database', None),
             ('Owner', 'root'),
             ('Created Time', 'Wed Feb 04 18:15:00 UTC 1815'),
             ('Last Access', 'Wed May 20 19:25:00 UTC 1925'),
@@ -136,7 +137,7 @@ class TestSparkAdapter(unittest.TestCase):
         rows = SparkAdapter(config).parse_describe_extended(relation, input_cols)
         self.assertEqual(len(rows), 3)
         self.assertEqual(rows[0].to_dict(omit_none=False), {
-            'table_database': relation.database,
+            'table_database': None,
             'table_schema': relation.schema,
             'table_name': relation.name,
             'table_type': rel_type,
@@ -150,7 +151,7 @@ class TestSparkAdapter(unittest.TestCase):
         })
 
         self.assertEqual(rows[1].to_dict(omit_none=False), {
-            'table_database': relation.database,
+            'table_database': None,
             'table_schema': relation.schema,
             'table_name': relation.name,
             'table_type': rel_type,
@@ -164,7 +165,7 @@ class TestSparkAdapter(unittest.TestCase):
         })
 
         self.assertEqual(rows[2].to_dict(omit_none=False), {
-            'table_database': relation.database,
+            'table_database': None,
             'table_schema': relation.schema,
             'table_name': relation.name,
             'table_type': rel_type,
@@ -182,11 +183,11 @@ class TestSparkAdapter(unittest.TestCase):
         rel_type = SparkRelation.get_relation_type.Table
 
         relation = SparkRelation.create(
-            database='default_database',
             schema='default_schema',
             identifier='mytable',
             type=rel_type
         )
+        assert relation.database is None
 
         # Mimics the output of Spark with a DESCRIBE TABLE EXTENDED
         plain_rows = [
@@ -194,7 +195,7 @@ class TestSparkAdapter(unittest.TestCase):
             ('# Partition Information', 'data_type'),
             (None, None),
             ('# Detailed Table Information', None),
-            ('Database', relation.database),
+            ('Database', None),
             ('Owner', 'root'),
             ('Created Time', 'Wed Feb 04 18:15:00 UTC 1815'),
             ('Last Access', 'Wed May 20 19:25:00 UTC 1925'),
@@ -214,7 +215,7 @@ class TestSparkAdapter(unittest.TestCase):
         rows = SparkAdapter(config).parse_describe_extended(relation, input_cols)
         self.assertEqual(len(rows), 1)
         self.assertEqual(rows[0].to_dict(omit_none=False), {
-            'table_database': relation.database,
+            'table_database': None,
             'table_schema': relation.schema,
             'table_name': relation.name,
             'table_type': rel_type,
@@ -236,3 +237,33 @@ class TestSparkAdapter(unittest.TestCase):
             'stats:rows:label': 'rows',
             'stats:rows:value': 14093476,
         })
+
+    def test_relation_with_database(self):
+        config = self._get_target_http(self.project_cfg)
+        adapter = SparkAdapter(config)
+        # fine
+        adapter.Relation.create(schema='different', identifier='table')
+        with self.assertRaises(RuntimeException):
+            # not fine - database set
+            adapter.Relation.create(database='something', schema='different', identifier='table')
+
+    def test_profile_with_database(self):
+        profile = {
+            'outputs': {
+                'test': {
+                    'type': 'spark',
+                    'method': 'http',
+                    # not allowed
+                    'database': 'analytics2',
+                    'schema': 'analytics',
+                    'host': 'myorg.sparkhost.com',
+                    'port': 443,
+                    'token': 'abc123',
+                    'organization': '0123456789',
+                    'cluster': '01234-23423-coffeetime',
+                }
+            },
+            'target': 'test'
+        }
+        with self.assertRaises(RuntimeException):
+            config_from_parts_or_dicts(self.project_cfg, profile)
