@@ -35,13 +35,14 @@ def mock_connection(name):
 
 
 def profile_from_dict(profile, profile_name, cli_vars='{}'):
-    from dbt.config import Profile, ConfigRenderer
+    from dbt.config import Profile
+    from dbt.config.renderer import ProfileRenderer
     from dbt.context.base import generate_base_context
     from dbt.utils import parse_cli_vars
     if not isinstance(cli_vars, dict):
         cli_vars = parse_cli_vars(cli_vars)
 
-    renderer = ConfigRenderer(generate_base_context(cli_vars))
+    renderer = ProfileRenderer(generate_base_context(cli_vars))
     return Profile.from_raw_profile_info(
         profile,
         profile_name,
@@ -51,12 +52,13 @@ def profile_from_dict(profile, profile_name, cli_vars='{}'):
 
 def project_from_dict(project, profile, packages=None, cli_vars='{}'):
     from dbt.context.target import generate_target_context
-    from dbt.config import Project, ConfigRenderer
+    from dbt.config import Project
+    from dbt.config.renderer import DbtProjectYamlRenderer
     from dbt.utils import parse_cli_vars
     if not isinstance(cli_vars, dict):
         cli_vars = parse_cli_vars(cli_vars)
 
-    renderer = ConfigRenderer(generate_target_context(profile, cli_vars))
+    renderer = DbtProjectYamlRenderer(generate_target_context(profile, cli_vars))
 
     project_root = project.pop('project-root', os.getcwd())
 
